#!/bin/bash

pscr=$(basename $0)
cscr=$(basename ${BASH_SOURCE})

# 以下 ##Cの行のコメントをはずせば、gpioグループの動作確認ができる
##C id

### LED Control ###########################
nled=4  # LEDの数
led0=(22 17 27)
led1=(25 23 24)
led2=(4 2 3)
led3=(18 14 15)

# led制御できるか確認、できなければled機能はoff
st=0
##C echo "${led0[0]} > /sys/class/gpio/export "
echo "${led0[0]}" > /sys/class/gpio/export
let st+=$?
sleep 0.1	# これがないと、sudoしないときに、うまくいかない
##C ls -l /sys/class/gpio
##C echo "out > /sys/class/gpio/gpio${led0[0]}/direction"
echo "out" > /sys/class/gpio/gpio${led0[0]}/direction
let st+=$?
##C ls -l /sys/class/gpio
##C echo "1 > /sys/class/gpio/gpio${led0[0]}/value"
echo "1" > /sys/class/gpio/gpio${led0[0]}/value
let st+=$?
##C ls -l /sys/class/gpio
##C echo "0 > /sys/class/gpio/gpio${led0[0]}/value"
echo "0" > /sys/class/gpio/gpio${led0[0]}/value
let st+=$?
##C ls -l /sys/class/gpio
##C echo "${led0[0]} > /sys/class/gpio/unexport"
echo "${led0[0]}" > /sys/class/gpio/unexport
let st+=$?
##C ls -l /sys/class/gpio

##C echo "status was $st"
##C exit

if [ $st -gt 0 ]; then # gpioの制御に失敗
  if [ "$pscr" = "$cscr" ]; then # 単独で実行
    exit $st
  else                           # sourceされている
    return $st
  fi
fi

led () { # LED Colors
    # 第1引数に led番号 以降に色(r,g,b)を任意に組み合わせて指定
    # 黒 引数無し/ 赤 r/ 黄 r g/ 紫 r b/ 緑 g/ 水 g b/ 青 b/ 白 r g b 
    [ $dark -eq 1 ] && return
    local l=$1
    [ $l -ge $nled -o $l -lt 0 ] && return
    local color=(0 0 0)
    local c port
    shift
    while [ "$1" ]
    do
        case "$1" in
          [rR]) color[0]=1; ;;
          [bB]) color[1]=1; ;;
          [gG]) color[2]=1; ;;
        esac
        shift
    done 

    for ((c=0; c<3; c++))
	{
        #echo "$l:$c"
        eval port=\${led$l[$c]}
        echo "${color[${c}]}" > /sys/class/gpio/gpio${port}/value
    }
}

led_open_close () {
    [ $dark -eq 1 ] && return
    [ $1 -ge $nled -o $1 -lt 0 ] && return
    local l=$1
    local c=$2
    local o=$3
    local port
	eval port=\${led$l[$c]}
    if [ $o -eq 1 ]; then
        echo "$port" > /sys/class/gpio/export
		sleep 0.1	# これがないと、sudoしないときに、うまくいかない
        echo "out" > /sys/class/gpio/gpio${port}/direction
    else
        echo "$port" > /sys/class/gpio/unexport
    fi
}


led_sw () { # LEDのon:1, off:0
    [ $dark -eq 1 ] && return
    local o=$1
    local l c port
    for ((l=0; l<nled; l++))
    {
        for ((c=0; c<3; c++))
        {
            if [ $o -eq 1 ]; then
                led_open_close $l $c 1
                eval port=\${led$l[$c]}
                echo "1" > /sys/class/gpio/gpio${port}/value || return 1
            else
                eval port=\${led$l[$c]}
                echo "0" > /sys/class/gpio/gpio${port}/value || return 1
                led_open_close $l $c 0
            fi
        }
    }
}


led_action () {
    [ $dark -eq 1 ] && return
    local i l c
    for ((i=0; i<16; i++))
    {
        for ((l=0; l<nled; l++))
        {
            led $l "r" "g" "b"
            #sleepenh 0.02 > /dev/null 2>&1 
            sleep 0.02

            led $l
            sleep 0.02
        }
        sleep 0.04
    }
    sleep 0.32
}
### END LED Control ###########################

### Stand alone main function
if [ "$pscr" = "$cscr" ]; then
    trap 'echo " exiting.." ; sleep 2 ; led_sw 0 ; exit' 2
    dark=0

    led_sw 1
    sleep 2
    led_action
    echo "------------------------"

    echo "LED0=R B"
    led 0 "r" "b"
    sleep 1

    echo "LED1=R"
    led 1 "r"
    sleep 1

    echo "LED2=G"
    led 2 "g"
    sleep 1

    echo "LED3=B"
    led 3 "b"

    sleep 5
    echo "------------------------"
    led_sw 0
fi

unset st
