#!/bin/bash

# Functions 1

message () {
    [ ${quiet} -eq 1 ] || echo "$@"
}

usage_exit() {
  echo "Usage: $0  [-?] [-l] [-q] [-d] [-i INTERVAL(sec)]  [servers ...]" 1>&2
 cat << USAGE

 mode and options :
    -l: Flush log file
    -i: set interval (default ${interval}
    -q: Quiet mode; no standard output
    -d: Dark mode; no led indication
    -?: Help
USAGE
  exit 1
}

## abstract functions
led() {
    :
}

led_sw() {
    :
}

led_action() {
    :
}
##

# Initializations - 初期化

interval=300       # 監視間隔(sec)
log="./shikatsu-log.txt" # logファイル

fgrep -i "raspbian" /etc/issue > /dev/null 2>&1
rpi=$?
led_script=$(cd $(dirname $0); pwd)"/led_pi.bash"
if [ ${rpi} -eq 0 -a -e ${led_script} ]; then
    echo "Raspberry Pi!"
    echo "reading ${led_script}"
    . ${led_script}
	if [ $? -gt 0 ]; then echo "No LED functions are available."; fi
else
    echo "This is not a Raspberry Pi. LED functions are disabled."
fi
##

quiet=0
dark=0
while getopts i:dlq OPT
do
  case $OPT in
    i) interval=$(expr ${OPTARG} + 0) > /dev/null 2>&1
		if [ $? -gt 0 ]; then
			interval=300
		elif [ ${interval} -lt 30 ]; then
			interval=30
		fi
		;;
    d) dark=1; ;;
    q) quiet=1; ;;
    l) echo > $log; echo "${log} Flushed"; ;;
    \?) usage_exit ;;
  esac
done
shift $((OPTIND - 1))

target=()   # 監視対象
if [ $# -eq 0 ] ; then
	# デフォルトの監視対象
    target=(www.aaabbbccc.net fs.dddeeefff.com mail.ggghhhiii.jp sss.jjjkkklll.io)
else  # 監視対象をコマンド引数で受け取る
    n=$#
    for ((i=0; i<n; i++))
    {
        target+=($1)
        shift
    }
fi

stat=() # 各サーバーの状態
ppid=()  # 各pingのPID
pid=()  # アニメーションのpid
cstat=()    # サーバー状態による表示色
ndown=()    # 各サーバーのダウンした回数
cdown=()    # 各サーバーのダウン頻度による表示色

nocursor="\033[?25l\033[37;0m"  # カーソルを表示しないエスケープシーケンス

# 開始時の情報表示 & Start logging & 配列の初期化
echo >> $log
message $(date) "++ shikatsu start ++"
echo $(date) "++ shikatsu start ++" >> $log
for i in "${target[@]}"
do
    message -n "$i "
    echo -n "$i " >> $log
    stat+=(0)     # 現在状態
    cstat+=("37") # 状態による表示色
    ndown+=(0)    # ダウンした回数
    cdown+=("")   # ダウン頻度による表示属性
done
message
echo >> $log
message "Watching ${#target[@]} servers."
echo "Watching ${#target[@]} servers." >> $log
message "Log file: ${log}"
message "Interval : ${interval}(sec)"
echo "Interval : ${interval}(sec)" >> $log
message
echo >> $log

message -e ${nocursor}
message -ne "---> Starting..\r"
sleep 1
###
led_sw 1

# CTRL-C 割り込み時に終了する処理
trap 'echo " exiting" ; echo $(date) "-- shikatsu stoped --" >> $log; for p in ${ppid[@]} ${pid[@]}; do wait $p ; done ; reset ; led_sw 0 ; exit' 2

## Functions 2
downcolor() { # ダウン頻度での表示属性変更
    n=$1
    if   [ ${ndown[$n]} -gt 7 ] ; then
        cdown[$n]=";7;4"
    elif [ ${ndown[$n]} -gt 3  ] ; then
        cdown[$n]=";7"
    elif [ ${ndown[$n]} -gt 1  ] ; then
        cdown[$n]=";4"
    else 
        cdown[$n]=""
    fi
}

chr_action () { # ping待ちでの表示演出
    dot=("     " ".    " "..   " "... " "....")
    t=$1
    n=$2
    for ((j=0; j<t; j++))
    {
        c=$(( RANDOM % 7 + 31)) # 色:$RANDOMである必要はない

        for ((i=n-1; i>=0; i--))
        {
            b=$(($i % 5))
            message -ne " "
            if [ $t -eq 1 ] ; then
                if [ ${i} -gt 9 ]; then
                    message -ne "   ${i}${dot[${b}]}\r"
                else
                    message -ne "    ${i}${dot[${b}]}\r"
                fi
            else
                message -ne "\033[${c}m* "
            fi
            sleep 1
        }
        message -ne "\r                \r"
    } # このループで $1 ×$2 (sec) sleep する
}

display_server_name() { # 対象サーバーの表示
	message -ne "\r                                                                   \r"
    for ((i=0; i<${#target[@]}; i++))
    {
        message -ne "\033[37;0m(${ndown[$i]}):\033[${cstat[$i]}${cdown[$i]}m${target[$i]}\033[0m\t"
        if [ $dark -ne 1 ]; then
            case ${cstat[$i]} in
              31 ) led $i "R"; ;;
              32 ) led $i "G"; ;;
              37 ) led $i "B"; ;;
            esac
        fi
    }
    message
}

status_on_name() { # 状態による表示色・属性の変更
    n=$1
    s=$2

    case "${stat[$n]}" in
        "0" ) # "前回生きていた"
            if [ $s -eq 0 ]; then # B "生きている"
                stat[$n]=0
                cstat[$n]="37"
            else # R "死んでしまった"
                stat[$n]=2
                cstat[$n]="31"
                ndown[$n]=$((${ndown[$n]} + 1))
                downcolor $n
                echo $(date) " DOWN      ${target[$n]}:${ndown[$n]}" >> $log
            fi
            ;;
        "1" ) # "復活した"
            if [ $s -eq 0 ]; then # B "生きている"
                stat[$n]=0
                cstat[$n]="37"
            else # R "死んでしまった"
                stat[$n]=2
                cstat[$n]="31"
                ndown[$n]=$((${ndown[$n]} + 1))
                downcolor $n
                echo $(date) " DOWN      ${target[$n]}:${ndown[$n]}" >> $log
            fi
            ;;
        "2" ) # "前回死んでいた"
            if [ $s -eq 0 ]; then # G "復活"
                stat[$n]=1
                cstat[$n]="32"
                echo $(date) " RECOVERED ${target[$n]}" >> $log
            else # R "死んだまま"
                stat[$n]=2
                cstat[$n]="31"
            fi
            ;;
    esac
}

# mainループ
count=0 # 監視ループの回数 (内部状態)
while :
do
    let count++
    for ((i=0; i<${#target[@]}; i++))
    {
        ping -c 1 ${target[$i]} > /dev/null 2>&1 &
        ppid+=($!)
    }

    chr_action 2 5  & # ping中の演出 2x5 (sec)
    pid+=($!)     # functionの呼び出し、pidが入る
    led_action &
    pid+=($!)     # functionの呼び出し、pidが入る

    for p in ${pid[@]} ; do wait $p ; done
    pid=()

    i=0
    for p in ${ppid[@]}
    do
        wait $p
        status_on_name $i $?
        let i++
    done
    ppid=()

    display_server_name

    message -ne "\t\tLast update:" $(date) "\r"
    chr_action 1 $interval # 1 x interval (sec)
done

